JacobiSymbol

Enum JacobiSymbol 

#[repr(i8)]
pub enum JacobiSymbol { Zero = 0, One = 1, MinusOne = -1, }
Possible return values for Jacobi symbol calculations.

Variants



Zero = 0

The two arguments are not coprime, they have a common divisor apart from 1.



One = 1

The two arguments are coprime. If the lower argument is prime, then the upper argument is quadratic residue modulo the lower argument. Otherwise, the upper argument is known to be quadratic nonresidue for an even number of prime factors of the lower argument.



MinusOne = -1

The two terms are coprime, and the upper argument is a quadratic nonresidue modulo the lower argument.

Implementations

Source

impl JacobiSymbol

pub const fn is_zero(&self) -> Choice

Determine if the symbol is zero.

pub const fn is_one(&self) -> Choice

Determine if the symbol is one.

pub const fn is_minus_one(&self) -> Choice

Determine if the symbol is minus one.

pub const fn neg(self) -> Self

Negate the symbol.