pub enum RandomBitsError<T> {
RandCore(T),
BitsPrecisionMismatch {
bits_precision: u32,
integer_bits: u32,
},
BitLengthTooLarge {
bit_length: u32,
bits_precision: u32,
},
}Available on crate feature
rand_core only.Expand description
Possible errors of the methods in RandomBits trait.
Variants
RandCore(T)
An error of the internal RNG library.
BitsPrecisionMismatch
The requested bits_precision does not match the size of the integer
corresponding to the type (in the cases where this is set in compile time).
Fields
BitLengthTooLarge
The requested bit_length is larger than bits_precision.