PrimeParams

Struct PrimeParams 

pub struct PrimeParams<const LIMBS: usize> { /* private fields */ }
Parameters for supporting efficient computations on integers in Montgomery form with a prime modulus.

Implementations

Source

impl<const LIMBS: usize> PrimeParams<LIMBS>

pub const fn new_vartime( params: &FixedMontyParams<LIMBS>, generator: u32, ) -> Self

Instantiates a new set of PrimeParams given FixedMontyParams for a prime modulus.

The value generator must be a multiplicative generator (ie. primitive element) of the finite field, having order modulus-1. Its powers generate all nonzero elements of the field: generator^0, generator^1, ..., generator^(modulus-2) enumerate [1, modulus-1].

pub const fn generator(&self) -> NonZeroU32

Get the constant ‘generator’ used in modular square root calculation.

pub const fn s(&self) -> NonZeroU32

Get the constant ‘s’ used in modular square root calculation.

pub const fn t(&self) -> OddUint<LIMBS>

Get the constant ‘t’ used in modular square root calculation.