Type Alias OddInt
Source pub type OddInt<const LIMBS: usize> = Odd<Int<LIMBS>>;
pub struct OddInt<const LIMBS: usize>();
Output type.
Compute the greatest common divisor of self and rhs.
Compute the greatest common divisor of self and rhs in variable time.
Output type.
Compute the greatest common divisor of self and rhs.
Compute the greatest common divisor of self and rhs in variable time.
Output type.
Compute the greatest common divisor of self and rhs.
Compute the greatest common divisor of self and rhs in variable time.
Output type.
Compute the extended greatest common divisor of self and rhs.
Compute the extended greatest common divisor of self and rhs in variable time.