#[repr(i8)]pub enum JacobiSymbol {
Zero = 0,
One = 1,
MinusOne = -1,
}Expand description
Possible return values for Jacobi symbol calculations.
Variants
Zero = 0
The two arguments are not coprime, they have a common divisor apart from 1.
One = 1
The two arguments are coprime. If the lower argument is prime, then the upper argument is quadratic residue modulo the lower argument. Otherwise, the upper argument is known to be quadratic nonresidue for an even number of prime factors of the lower argument.
MinusOne = -1
The two terms are coprime, and the upper argument is a quadratic nonresidue modulo the lower argument.
Implementations
Sourceimpl JacobiSymbol
impl JacobiSymbol