pub struct BoxedMontyForm { /* private fields */ }alloc only.Expand description
An integer in Montgomery form represented using heap-allocated limbs.
Implementations
Sourceimpl BoxedMontyForm
impl BoxedMontyForm
Sourcepub fn invert(&self) -> CtOption<Self>
pub fn invert(&self) -> CtOption<Self>
Computes self^-1 representing the multiplicative inverse of self,
i.e. self * self^-1 = 1.
Sourcepub fn invert_vartime(&self) -> CtOption<Self>
pub fn invert_vartime(&self) -> CtOption<Self>
Computes self^-1 representing the multiplicative inverse of self,
i.e. self * self^-1 = 1.
This version is variable-time with respect to the self of self, but constant-time with
respect to self’s params.
Sourceimpl BoxedMontyForm
impl BoxedMontyForm
Sourcepub fn lincomb_vartime(products: &[(&Self, &Self)]) -> Self
pub fn lincomb_vartime(products: &[(&Self, &Self)]) -> Self
Calculate the sum of products of pairs (a, b) in products.
This method is variable time only with the value of the modulus.
For a modulus with leading zeros, this method is more efficient than a naive sum of products.
All terms must be associated with equivalent MontyParams.
Panics
- if
productsis empty.
Sourceimpl BoxedMontyForm
impl BoxedMontyForm
Sourcepub fn pow_bounded_exp(&self, exponent: &BoxedUint, exponent_bits: u32) -> Self
pub fn pow_bounded_exp(&self, exponent: &BoxedUint, exponent_bits: u32) -> Self
Raises to the exponent power,
with exponent_bits representing the number of (least significant) bits
to take into account for the exponent.
NOTE: exponent_bits may be leaked in the time pattern.
Sourceimpl BoxedMontyForm
impl BoxedMontyForm
Sourcepub fn new(integer: BoxedUint, params: &BoxedMontyParams) -> Self
pub fn new(integer: BoxedUint, params: &BoxedMontyParams) -> Self
Instantiates a new BoxedMontyForm that represents an integer modulo the provided params.
Sourcepub fn bits_precision(&self) -> u32
pub fn bits_precision(&self) -> u32
Bits of precision in the modulus.
Sourcepub fn retrieve(&self) -> BoxedUint
pub fn retrieve(&self) -> BoxedUint
Retrieves the integer currently encoded in this BoxedMontyForm, guaranteed to be reduced.
Sourcepub fn zero(params: &BoxedMontyParams) -> Self
pub fn zero(params: &BoxedMontyParams) -> Self
Instantiates a new ConstMontyForm that represents zero.
Sourcepub fn one(params: &BoxedMontyParams) -> Self
pub fn one(params: &BoxedMontyParams) -> Self
Instantiates a new ConstMontyForm that represents 1.
Sourcepub fn is_zero(&self) -> Choice
pub fn is_zero(&self) -> Choice
Determine if this value is equal to zero.
Returns
If zero, returns Choice(1). Otherwise, returns Choice(0).
Sourcepub fn is_nonzero(&self) -> Choice
pub fn is_nonzero(&self) -> Choice
Determine if this value is not equal to zero.
Returns
If zero, returns Choice(0). Otherwise, returns Choice(1).
Sourcepub fn params(&self) -> &BoxedMontyParams
pub fn params(&self) -> &BoxedMontyParams
Returns the parameter struct used to initialize this object.
Sourcepub fn as_montgomery(&self) -> &BoxedUint
pub fn as_montgomery(&self) -> &BoxedUint
Access the BoxedMontyForm value in Montgomery form.
Sourcepub fn as_montgomery_mut(&mut self) -> &mut BoxedUint
pub fn as_montgomery_mut(&mut self) -> &mut BoxedUint
Mutably access the BoxedMontyForm value in Montgomery form.
Sourcepub fn from_montgomery(integer: BoxedUint, params: &BoxedMontyParams) -> Self
pub fn from_montgomery(integer: BoxedUint, params: &BoxedMontyParams) -> Self
Create a BoxedMontyForm from a value in Montgomery form.
Sourcepub fn to_montgomery(&self) -> BoxedUint
pub fn to_montgomery(&self) -> BoxedUint
Extract the value from the BoxedMontyForm in Montgomery form.
Sourcepub fn div_by_2(&self) -> Self
pub fn div_by_2(&self) -> Self
Performs division by 2, that is returns x such that x + x = self.
Sourcepub fn div_by_2_assign(&mut self)
pub fn div_by_2_assign(&mut self)
Performs division by 2 inplace, that is finds x such that x + x = self
and writes it into self.