pub struct ConstMontyForm<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> { /* private fields */ }Expand description
An integer in Montgomery form modulo MOD, represented using LIMBS limbs.
The modulus is constant, so it cannot be set at runtime.
Internally, the value is stored in Montgomery form (multiplied by MOD::PARAMS.one) until it is retrieved.
Implementations
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourcepub const fn add(&self, rhs: &ConstMontyForm<MOD, LIMBS>) -> Self
pub const fn add(&self, rhs: &ConstMontyForm<MOD, LIMBS>) -> Self
Adds rhs.
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourcepub const fn inv(&self) -> CtOption<Self>
👎Deprecated since 0.7.0: please use invert instead
pub const fn inv(&self) -> CtOption<Self>
please use invert instead
Computes self^-1 representing the multiplicative inverse of self,
i.e. self * self^-1 = 1.
If the number was invertible, the second element of the tuple is the truthy value, otherwise it is the falsy value (in which case the first element’s value is unspecified).
Sourcepub const fn invert(&self) -> CtOption<Self>
pub const fn invert(&self) -> CtOption<Self>
Computes self^-1 representing the multiplicative inverse of self,
i.e. self * self^-1 = 1.
If the number was invertible, the second element of the tuple is the truthy value, otherwise it is the falsy value (in which case the first element’s value is unspecified).
Sourcepub const fn inv_vartime(&self) -> CtOption<Self>
👎Deprecated since 0.7.0: please use invert_vartime instead
pub const fn inv_vartime(&self) -> CtOption<Self>
please use invert_vartime instead
Computes self^-1 representing the multiplicative inverse of self,
i.e. self * self^-1 = 1.
If the number was invertible, the second element of the tuple is the truthy value, otherwise it is the falsy value (in which case the first element’s value is unspecified).
This version is variable-time with respect to the value of self, but constant-time with
respect to MOD.
Sourcepub const fn invert_vartime(&self) -> CtOption<Self>
pub const fn invert_vartime(&self) -> CtOption<Self>
Computes self^-1 representing the multiplicative inverse of self,
i.e. self * self^-1 = 1.
If the number was invertible, the second element of the tuple is the truthy value, otherwise it is the falsy value (in which case the first element’s value is unspecified).
This version is variable-time with respect to the value of self, but constant-time with
respect to MOD.
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourcepub const fn jacobi_symbol(&self) -> JacobiSymbol
pub const fn jacobi_symbol(&self) -> JacobiSymbol
Compute the Jacobi symbol (self|modulus).
For a prime modulus, this corresponds to the Legendre symbol and indicates
whether self is quadratic residue.
Sourcepub const fn jacobi_symbol_vartime(&self) -> JacobiSymbol
pub const fn jacobi_symbol_vartime(&self) -> JacobiSymbol
Compute the Jacobi symbol (self|modulus).
For a prime modulus, this corresponds to the Legendre symbol and indicates
whether self is quadratic residue.
This method is variable-time with respect to the value of self.
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourcepub const fn pow<const RHS_LIMBS: usize>(
&self,
exponent: &Uint<RHS_LIMBS>,
) -> Self
pub const fn pow<const RHS_LIMBS: usize>( &self, exponent: &Uint<RHS_LIMBS>, ) -> Self
Raises to the exponent power.
Sourcepub const fn pow_bounded_exp<const RHS_LIMBS: usize>(
&self,
exponent: &Uint<RHS_LIMBS>,
exponent_bits: u32,
) -> Self
pub const fn pow_bounded_exp<const RHS_LIMBS: usize>( &self, exponent: &Uint<RHS_LIMBS>, exponent_bits: u32, ) -> Self
Raises to the exponent power,
with exponent_bits representing the number of (least significant) bits
to take into account for the exponent.
NOTE: exponent_bits may be leaked in the time pattern.
Sourcepub const fn pow_vartime<const RHS_LIMBS: usize>(
&self,
exponent: &Uint<RHS_LIMBS>,
) -> Self
pub const fn pow_vartime<const RHS_LIMBS: usize>( &self, exponent: &Uint<RHS_LIMBS>, ) -> Self
Raises to the exponent power.
This method is variable time in exponent.
Sourcepub fn pow_amm(&self, exponent: &Uint<LIMBS>) -> Self
pub fn pow_amm(&self, exponent: &Uint<LIMBS>) -> Self
Raises to the exponent power using Almost Montgomery Multiplication (AMM).
Sourcepub fn pow_amm_bounded_exp(
&self,
exponent: &Uint<LIMBS>,
exponent_bits: u32,
) -> Self
pub fn pow_amm_bounded_exp( &self, exponent: &Uint<LIMBS>, exponent_bits: u32, ) -> Self
Raises to the exponent power using Almost Montgomery Multiplication (AMM)
with exponent_bits representing the number of (least significant) bits
to take into account for the exponent.
NOTE: exponent_bits may be leaked in the time pattern.
Sourceimpl<const LIMBS: usize, MOD> ConstMontyForm<MOD, LIMBS>where
MOD: ConstPrimeMontyParams<LIMBS>,
impl<const LIMBS: usize, MOD> ConstMontyForm<MOD, LIMBS>where
MOD: ConstPrimeMontyParams<LIMBS>,
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourceimpl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
impl<MOD: ConstMontyParams<LIMBS>, const LIMBS: usize> ConstMontyForm<MOD, LIMBS>
Sourcepub const fn new(integer: &Uint<LIMBS>) -> Self
pub const fn new(integer: &Uint<LIMBS>) -> Self
Instantiates a new ConstMontyForm that represents this integer mod MOD.
Sourcepub const fn retrieve(&self) -> Uint<LIMBS>
pub const fn retrieve(&self) -> Uint<LIMBS>
Retrieves the integer currently encoded in this ConstMontyForm, guaranteed to be reduced.
Sourcepub const fn as_montgomery(&self) -> &Uint<LIMBS>
pub const fn as_montgomery(&self) -> &Uint<LIMBS>
Access the ConstMontyForm value in Montgomery form.
Sourcepub fn as_montgomery_mut(&mut self) -> &mut Uint<LIMBS>
pub fn as_montgomery_mut(&mut self) -> &mut Uint<LIMBS>
Mutably access the ConstMontyForm value in Montgomery form.
Sourcepub const fn from_montgomery(integer: Uint<LIMBS>) -> Self
pub const fn from_montgomery(integer: Uint<LIMBS>) -> Self
Create a ConstMontyForm from a value in Montgomery form.
Sourcepub const fn to_montgomery(&self) -> Uint<LIMBS>
pub const fn to_montgomery(&self) -> Uint<LIMBS>
Extract the value from the ConstMontyForm in Montgomery form.