pub struct FixedMontyForm<const LIMBS: usize> { /* private fields */ }Expand description
An integer in Montgomery form represented using LIMBS limbs.
The odd modulus is set at runtime.
Implementations
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourcepub const fn inv(&self) -> CtOption<Self>
👎Deprecated since 0.7.0: please use invert instead
pub const fn inv(&self) -> CtOption<Self>
invert insteadComputes self^-1 representing the multiplicative inverse of self.
i.e. self * self^-1 = 1.
If the number was invertible, the second element of the tuple is the truthy value, otherwise it is the falsy value (in which case the first element’s value is unspecified).
Sourcepub const fn invert(&self) -> CtOption<Self>
pub const fn invert(&self) -> CtOption<Self>
Computes self^-1 representing the multiplicative inverse of self.
i.e. self * self^-1 = 1.
If the number was invertible, the second element of the tuple is the truthy value, otherwise it is the falsy value (in which case the first element’s value is unspecified).
Sourcepub const fn inv_vartime(&self) -> CtOption<Self>
👎Deprecated since 0.7.0: please use invert_vartime instead
pub const fn inv_vartime(&self) -> CtOption<Self>
invert_vartime insteadComputes self^-1 representing the multiplicative inverse of self.
i.e. self * self^-1 = 1.
If the number was invertible, the second element of the tuple is the truthy value, otherwise it is the falsy value (in which case the first element’s value is unspecified).
This version is variable-time with respect to the value of self, but constant-time with
respect to self’s params.
Sourcepub const fn invert_vartime(&self) -> CtOption<Self>
pub const fn invert_vartime(&self) -> CtOption<Self>
Computes self^-1 representing the multiplicative inverse of self.
i.e. self * self^-1 = 1.
If the number was invertible, the second element of the tuple is the truthy value, otherwise it is the falsy value (in which case the first element’s value is unspecified).
This version is variable-time with respect to the value of self, but constant-time with
respect to self’s params.
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourcepub const fn lincomb_vartime(products: &[(&Self, &Self)]) -> Self
pub const fn lincomb_vartime(products: &[(&Self, &Self)]) -> Self
Calculate the sum of products of pairs (a, b) in products.
This method is variable time only with the value of the modulus. For a modulus with leading zeros, this method is more efficient than a naive sum of products.
This method will panic if products is empty. All terms must be associated
with equivalent MontyParams.
Panics
- if
productsis empty.
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourcepub const fn jacobi_symbol(&self) -> JacobiSymbol
pub const fn jacobi_symbol(&self) -> JacobiSymbol
Compute the Jacobi symbol (self|modulus).
For a prime modulus, this corresponds to the Legendre symbol and indicates
whether self is quadratic residue.
Sourcepub const fn jacobi_symbol_vartime(&self) -> JacobiSymbol
pub const fn jacobi_symbol_vartime(&self) -> JacobiSymbol
Compute the Jacobi symbol (self|modulus).
For a prime modulus, this corresponds to the Legendre symbol and indicates
whether self is quadratic residue.
This method is variable-time with respect to the value of self.
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourcepub const fn pow<const RHS_LIMBS: usize>(
&self,
exponent: &Uint<RHS_LIMBS>,
) -> Self
pub const fn pow<const RHS_LIMBS: usize>( &self, exponent: &Uint<RHS_LIMBS>, ) -> Self
Raises to the exponent power.
Sourcepub const fn pow_bounded_exp<const RHS_LIMBS: usize>(
&self,
exponent: &Uint<RHS_LIMBS>,
exponent_bits: u32,
) -> Self
pub const fn pow_bounded_exp<const RHS_LIMBS: usize>( &self, exponent: &Uint<RHS_LIMBS>, exponent_bits: u32, ) -> Self
Raises to the exponent power,
with exponent_bits representing the number of (least significant) bits
to take into account for the exponent.
NOTE: exponent_bits may be leaked in the time pattern.
Sourcepub const fn pow_vartime<const RHS_LIMBS: usize>(
&self,
exponent: &Uint<RHS_LIMBS>,
) -> Self
pub const fn pow_vartime<const RHS_LIMBS: usize>( &self, exponent: &Uint<RHS_LIMBS>, ) -> Self
Raises to the exponent power.
This method is variable time in exponent.
Sourcepub fn pow_amm(&self, exponent: &Uint<LIMBS>) -> Self
pub fn pow_amm(&self, exponent: &Uint<LIMBS>) -> Self
Raises to the exponent power using Almost Montgomery Multiplication (AMM).
Sourcepub fn pow_amm_bounded_exp(
&self,
exponent: &Uint<LIMBS>,
exponent_bits: u32,
) -> Self
pub fn pow_amm_bounded_exp( &self, exponent: &Uint<LIMBS>, exponent_bits: u32, ) -> Self
Raises to the exponent power using Almost Montgomery Multiplication (AMM)
with exponent_bits representing the number of (least significant) bits
to take into account for the exponent.
NOTE: exponent_bits may be leaked in the time pattern.
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourceimpl<const LIMBS: usize> FixedMontyForm<LIMBS>
impl<const LIMBS: usize> FixedMontyForm<LIMBS>
Sourcepub const fn new(
integer: &Uint<LIMBS>,
params: &FixedMontyParams<LIMBS>,
) -> Self
pub const fn new( integer: &Uint<LIMBS>, params: &FixedMontyParams<LIMBS>, ) -> Self
Instantiates a new MontyForm that represents this integer mod MOD.
Sourcepub const fn retrieve(&self) -> Uint<LIMBS>
pub const fn retrieve(&self) -> Uint<LIMBS>
Retrieves the integer currently encoded in this MontyForm, guaranteed to be reduced.
Sourcepub const fn zero(params: &FixedMontyParams<LIMBS>) -> Self
pub const fn zero(params: &FixedMontyParams<LIMBS>) -> Self
Instantiates a new MontyForm that represents zero.
Sourcepub const fn one(params: &FixedMontyParams<LIMBS>) -> Self
pub const fn one(params: &FixedMontyParams<LIMBS>) -> Self
Instantiates a new MontyForm that represents 1.
Sourcepub const fn params(&self) -> &FixedMontyParams<LIMBS>
pub const fn params(&self) -> &FixedMontyParams<LIMBS>
Returns the parameter struct used to initialize this object.
Sourcepub const fn as_montgomery(&self) -> &Uint<LIMBS>
pub const fn as_montgomery(&self) -> &Uint<LIMBS>
Access the MontyForm value in Montgomery form.
Sourcepub fn as_montgomery_mut(&mut self) -> &mut Uint<LIMBS>
pub fn as_montgomery_mut(&mut self) -> &mut Uint<LIMBS>
Mutably access the MontyForm value in Montgomery form.
Sourcepub const fn from_montgomery(
integer: Uint<LIMBS>,
params: &FixedMontyParams<LIMBS>,
) -> Self
pub const fn from_montgomery( integer: Uint<LIMBS>, params: &FixedMontyParams<LIMBS>, ) -> Self
Create a MontyForm from a value in Montgomery form.
Sourcepub const fn to_montgomery(&self) -> Uint<LIMBS>
pub const fn to_montgomery(&self) -> Uint<LIMBS>
Extract the value from the MontyForm in Montgomery form.