pub struct MontyParams<U: Unsigned> { /* private fields */ }Expand description
Parameters to efficiently go to/from the Montgomery form for an odd modulus provided at runtime.
This version is generic over the underlying unsigned integer type.
Implementations
Sourceimpl<U: Unsigned> MontyParams<U>
impl<U: Unsigned> MontyParams<U>
Sourcepub const fn modulus(&self) -> &Odd<U>
pub const fn modulus(&self) -> &Odd<U>
Returns the modulus which was used to initialize these parameters.
Sourcepub const fn mod_inv(&self) -> &U64
pub const fn mod_inv(&self) -> &U64
Returns the lowest limbs of MODULUS^-1 mod 2**64.
This value is used in Montgomery reduction and modular inversion.
Sourcepub const fn mod_neg_inv(&self) -> Limb
pub const fn mod_neg_inv(&self) -> Limb
Returns wrapping negation of first limb of mod_inv.
Sourcepub const fn mod_leading_zeros(&self) -> u32
pub const fn mod_leading_zeros(&self) -> u32
Returns leading zeros in the modulus, used to choose optimized algorithms.
Sourceimpl<const LIMBS: usize> MontyParams<Uint<LIMBS>>
impl<const LIMBS: usize> MontyParams<Uint<LIMBS>>
Sourceimpl<const LIMBS: usize> MontyParams<Uint<LIMBS>>
impl<const LIMBS: usize> MontyParams<Uint<LIMBS>>
Sourcepub const fn new_vartime(modulus: Odd<Uint<LIMBS>>) -> Self
pub const fn new_vartime(modulus: Odd<Uint<LIMBS>>) -> Self
Instantiates a new set of MontyParams representing the given odd modulus.