#[repr(transparent)]pub struct Limb(pub Word);Expand description
Tuple Fields
0: WordImplementations
Sourceimpl Limb
impl Limb
Sourcepub const fn adc(self, rhs: Limb, carry: Limb) -> (Limb, Limb)
👎Deprecated since 0.7.0: please use carrying_add instead
pub const fn adc(self, rhs: Limb, carry: Limb) -> (Limb, Limb)
carrying_add insteadComputes self + rhs + carry, returning the result along with the new carry.
Sourcepub const fn carrying_add(self, rhs: Limb, carry: Limb) -> (Limb, Limb)
pub const fn carrying_add(self, rhs: Limb, carry: Limb) -> (Limb, Limb)
Computes self + rhs + carry, returning the result along with the new carry.
Sourcepub const fn overflowing_add(self, rhs: Limb) -> (Limb, Limb)
pub const fn overflowing_add(self, rhs: Limb) -> (Limb, Limb)
Computes self + rhs, returning the result along with the carry.
Sourcepub const fn saturating_add(&self, rhs: Self) -> Self
pub const fn saturating_add(&self, rhs: Self) -> Self
Perform saturating addition.
Sourcepub const fn wrapping_add(&self, rhs: Self) -> Self
pub const fn wrapping_add(&self, rhs: Self) -> Self
Perform wrapping addition, discarding overflow.
Sourceimpl Limb
impl Limb
Sourcepub const fn bit(self, index: u32) -> Choice
pub const fn bit(self, index: u32) -> Choice
Get the value of the bit at position index, as a truthy or falsy Choice.
Returns the falsy value for indices out of range.
Sourcepub const fn bit_vartime(self, index: u32) -> bool
pub const fn bit_vartime(self, index: u32) -> bool
Returns true if the bit at position index is set, false for an unset bit
or for indices out of range.
Remarks
This operation is variable time with respect to index only.
Sourcepub const fn leading_zeros(self) -> u32
pub const fn leading_zeros(self) -> u32
Calculate the number of leading zeros in the binary representation of this number.
Sourcepub const fn trailing_zeros(self) -> u32
pub const fn trailing_zeros(self) -> u32
Calculate the number of trailing zeros in the binary representation of this number.
Sourcepub const fn trailing_ones(self) -> u32
pub const fn trailing_ones(self) -> u32
Calculate the number of trailing ones the binary representation of this number.
Sourcepub const fn restrict_bits(self, len: u32) -> Self
pub const fn restrict_bits(self, len: u32) -> Self
Clear bits at or above the given bit position.
Sourceimpl Limb
impl Limb
Sourcepub fn cmp_vartime(self, other: Self) -> Ordering
pub fn cmp_vartime(self, other: Self) -> Ordering
Perform a comparison of the inner value in variable-time.
Note that the PartialOrd and Ord impls wrap constant-time
comparisons using the subtle crate.
Sourcepub const fn eq_vartime(self, other: Self) -> bool
pub const fn eq_vartime(self, other: Self) -> bool
Performs an equality check in variable-time.
Sourceimpl Limb
impl Limb
Sourcepub const fn div_rem(self, rhs: NonZero<Self>) -> (Limb, Limb)
pub const fn div_rem(self, rhs: NonZero<Self>) -> (Limb, Limb)
Computes self / rhs, returning the quotient and remainder.
Sourcepub const fn div_rem_with_reciprocal(self, recip: &Reciprocal) -> (Limb, Limb)
pub const fn div_rem_with_reciprocal(self, recip: &Reciprocal) -> (Limb, Limb)
Computes self / rhs where recip is a Reciprocal created from a non-zero Limb rhs.
Returns the quotient and remainder.
Sourcepub const fn checked_div(self, rhs: Self) -> CtOption<Limb>
pub const fn checked_div(self, rhs: Self) -> CtOption<Limb>
Computes the checked division self / rhs, returning the quotient
if the divisor is non-zero, and CtOption::none() otherwise.
Sourcepub const fn checked_rem(self, rhs: Self) -> CtOption<Limb>
pub const fn checked_rem(self, rhs: Self) -> CtOption<Limb>
Computes the checked division self / rhs, returning the remainder
if the divisor is non-zero, and CtOption::none() otherwise.
Sourceimpl Limb
impl Limb
Sourcepub const fn mac(self, b: Limb, c: Limb, carry: Limb) -> (Limb, Limb)
👎Deprecated since 0.7.0: please use carrying_mul_add instead (ordering of arguments changes)
pub const fn mac(self, b: Limb, c: Limb, carry: Limb) -> (Limb, Limb)
carrying_mul_add instead (ordering of arguments changes)Computes self + (b * c) + carry, returning the result along with the new carry.
Sourcepub const fn carrying_mul_add(
self,
rhs: Limb,
addend: Limb,
carry: Limb,
) -> (Limb, Limb)
pub const fn carrying_mul_add( self, rhs: Limb, addend: Limb, carry: Limb, ) -> (Limb, Limb)
Computes (self * rhs) + addend + carry, returning the result along with the new carry.
Sourcepub const fn saturating_mul(self, rhs: Self) -> Self
pub const fn saturating_mul(self, rhs: Self) -> Self
Perform saturating multiplication.
Sourcepub const fn wrapping_mul(self, rhs: Self) -> Self
pub const fn wrapping_mul(self, rhs: Self) -> Self
Perform wrapping multiplication, discarding overflow.
Sourceimpl Limb
impl Limb
Sourcepub const fn wrapping_neg(self) -> Self
pub const fn wrapping_neg(self) -> Self
Perform wrapping negation.
Sourceimpl Limb
impl Limb
Sourcepub const fn overflowing_shl(self, shift: u32) -> CtOption<Self>
pub const fn overflowing_shl(self, shift: u32) -> CtOption<Self>
Computes self << shift, returning CtOption::none() if the shift exceeds the capacity.
Sourcepub const fn overflowing_shl_vartime(self, shift: u32) -> Option<Self>
pub const fn overflowing_shl_vartime(self, shift: u32) -> Option<Self>
Computes self << shift, returning None if the shift exceeds the capacity.
This method is variable time in shift only.
Sourcepub const fn unbounded_shl(self, shift: u32) -> Self
pub const fn unbounded_shl(self, shift: u32) -> Self
Computes self << shift, returning Limb::ZERO if the shift exceeds the capacity.
Sourcepub const fn unbounded_shl_vartime(self, shift: u32) -> Self
pub const fn unbounded_shl_vartime(self, shift: u32) -> Self
Computes self << shift, returning Limb::ZERO if the shift exceeds the capacity.
This method is variable time in shift only.
Sourcepub const fn wrapping_shl(self, shift: u32) -> Self
pub const fn wrapping_shl(self, shift: u32) -> Self
Computes self << shift in a panic-free manner, masking off bits of shift
which would cause the shift to exceed the type’s width.
Sourceimpl Limb
impl Limb
Sourcepub const fn overflowing_shr(self, shift: u32) -> CtOption<Self>
pub const fn overflowing_shr(self, shift: u32) -> CtOption<Self>
Computes self >> shift, returning CtOption::none() if the shift exceeds the capacity.
Sourcepub const fn overflowing_shr_vartime(self, shift: u32) -> Option<Self>
pub const fn overflowing_shr_vartime(self, shift: u32) -> Option<Self>
Computes self >> shift, returning None if the shift exceeds the capacity.
This method is variable time in shift only.
Sourcepub const fn unbounded_shr(self, shift: u32) -> Self
pub const fn unbounded_shr(self, shift: u32) -> Self
Computes self >> shift, returning Limb::ZERO if the shift exceeds the capacity.
Sourcepub const fn unbounded_shr_vartime(self, shift: u32) -> Self
pub const fn unbounded_shr_vartime(self, shift: u32) -> Self
Computes self >> shift, returning Limb::ZERO if the shift exceeds the capacity.
This method is variable time in shift only.
Sourcepub const fn wrapping_shr(self, shift: u32) -> Self
pub const fn wrapping_shr(self, shift: u32) -> Self
Computes self >> shift in a panic-free manner, masking off bits of shift
which would cause the shift to exceed the type’s width.
Sourceimpl Limb
impl Limb
Sourcepub const fn sbb(self, rhs: Limb, borrow: Limb) -> (Limb, Limb)
👎Deprecated since 0.7.0: please use borrowing_sub instead
pub const fn sbb(self, rhs: Limb, borrow: Limb) -> (Limb, Limb)
borrowing_sub insteadComputes self - (rhs + borrow), returning the result along with the new borrow.
Sourcepub const fn borrowing_sub(self, rhs: Limb, borrow: Limb) -> (Limb, Limb)
pub const fn borrowing_sub(self, rhs: Limb, borrow: Limb) -> (Limb, Limb)
Computes self - (rhs + borrow), returning the result along with the new borrow.
Sourcepub const fn saturating_sub(&self, rhs: Self) -> Self
pub const fn saturating_sub(&self, rhs: Self) -> Self
Perform saturating subtraction.
Sourcepub const fn wrapping_sub(&self, rhs: Self) -> Self
pub const fn wrapping_sub(&self, rhs: Self) -> Self
Perform wrapping subtraction, discarding underflow and wrapping around the boundary of the type.
Sourceimpl Limb
impl Limb
Sourcepub const fn is_zero(&self) -> Choice
pub const fn is_zero(&self) -> Choice
Is this limb equal to Limb::ZERO?
Sourcepub const fn to_nz(self) -> CtOption<NonZero<Self>>
pub const fn to_nz(self) -> CtOption<NonZero<Self>>
Convert to a NonZero<Limb>.
Returns some if the original value is non-zero, and false otherwise.
Sourcepub const fn lsb_to_choice(self) -> Choice
pub const fn lsb_to_choice(self) -> Choice
Sourcepub const fn slice_as_words(slice: &[Self]) -> &[Word]
pub const fn slice_as_words(slice: &[Self]) -> &[Word]
Sourcepub const fn slice_as_mut_words(slice: &mut [Self]) -> &mut [Word]
pub const fn slice_as_mut_words(slice: &mut [Self]) -> &mut [Word]