pub struct NonZero<T: ?Sized>(/* private fields */);Expand description
Wrapper type for non-zero integers.
Implementations
Sourceimpl<const LIMBS: usize> NonZero<Int<LIMBS>>
impl<const LIMBS: usize> NonZero<Int<LIMBS>>
Sourcepub const fn gcd_unsigned(&self, rhs: &Uint<LIMBS>) -> NonZero<Uint<LIMBS>>
pub const fn gcd_unsigned(&self, rhs: &Uint<LIMBS>) -> NonZero<Uint<LIMBS>>
Compute the greatest common divisor of self and rhs.
Sourcepub const fn gcd_unsigned_vartime(
&self,
rhs: &Uint<LIMBS>,
) -> NonZeroUint<LIMBS>
pub const fn gcd_unsigned_vartime( &self, rhs: &Uint<LIMBS>, ) -> NonZeroUint<LIMBS>
Compute the greatest common divisor of self and rhs.
Executes in variable time w.r.t. all input parameters.
Sourcepub const fn xgcd(&self, rhs: &Self) -> XgcdOutput<LIMBS, NonZeroUint<LIMBS>>
pub const fn xgcd(&self, rhs: &Self) -> XgcdOutput<LIMBS, NonZeroUint<LIMBS>>
Execute the Extended GCD algorithm.
Given (self, rhs), computes (g, x, y) s.t. self * x + rhs * y = g = gcd(self, rhs).
Sourceimpl<T> NonZero<T>
impl<T> NonZero<T>
Sourcepub fn from_be_bytes(bytes: T::Repr) -> CtOption<Self>
pub fn from_be_bytes(bytes: T::Repr) -> CtOption<Self>
Decode from big endian bytes.
Sourcepub fn from_le_bytes(bytes: T::Repr) -> CtOption<Self>
pub fn from_le_bytes(bytes: T::Repr) -> CtOption<Self>
Decode from little endian bytes.
Sourceimpl NonZero<Limb>
impl NonZero<Limb>
Sourcepub const fn new_unwrap(n: Limb) -> Self
pub const fn new_unwrap(n: Limb) -> Self
Sourcepub const fn from_u8(n: NonZeroU8) -> Self
pub const fn from_u8(n: NonZeroU8) -> Self
Create a NonZero<Limb> from a NonZeroU8 (const-friendly)
Sourcepub const fn from_u16(n: NonZeroU16) -> Self
pub const fn from_u16(n: NonZeroU16) -> Self
Create a NonZero<Limb> from a NonZeroU16 (const-friendly)
Sourcepub const fn from_u32(n: NonZeroU32) -> Self
pub const fn from_u32(n: NonZeroU32) -> Self
Create a NonZero<Limb> from a NonZeroU32 (const-friendly)
Sourcepub const fn from_u64(n: NonZeroU64) -> Self
pub const fn from_u64(n: NonZeroU64) -> Self
Create a NonZero<Limb> from a NonZeroU64 (const-friendly)
Sourceimpl<const LIMBS: usize> NonZero<Uint<LIMBS>>
impl<const LIMBS: usize> NonZero<Uint<LIMBS>>
Sourcepub const fn new_unwrap(n: Uint<LIMBS>) -> Self
pub const fn new_unwrap(n: Uint<LIMBS>) -> Self
Creates a new non-zero integer in a const context.
In future versions of Rust it should be possible to replace this with
NonZero::new(…).unwrap()
Panics
- if the value is zero.
Sourcepub const fn from_be_hex(hex: &str) -> Self
pub const fn from_be_hex(hex: &str) -> Self
Create a new NonZero<Uint> from the provided big endian hex string.
Panics
- if the hex is zero, malformed, or not zero-padded accordingly for the size.
Sourcepub const fn from_le_hex(hex: &str) -> Self
pub const fn from_le_hex(hex: &str) -> Self
Create a new NonZero<Uint> from the provided little endian hex string.
Panics
- if the hex is zero, malformed, or not zero-padded accordingly for the size.
Sourcepub const fn from_u8(n: NonZeroU8) -> Self
pub const fn from_u8(n: NonZeroU8) -> Self
Create a NonZeroUint from a NonZeroU8 (const-friendly)
Sourcepub const fn from_u16(n: NonZeroU16) -> Self
pub const fn from_u16(n: NonZeroU16) -> Self
Create a NonZeroUint from a NonZeroU16 (const-friendly)
Sourcepub const fn from_u32(n: NonZeroU32) -> Self
pub const fn from_u32(n: NonZeroU32) -> Self
Create a NonZeroUint from a NonZeroU32 (const-friendly)
Sourcepub const fn from_u64(n: NonZeroU64) -> Self
pub const fn from_u64(n: NonZeroU64) -> Self
Create a NonZeroUint from a NonZeroU64 (const-friendly)
Sourcepub const fn from_u128(n: NonZeroU128) -> Self
pub const fn from_u128(n: NonZeroU128) -> Self
Create a NonZeroUint from a NonZeroU128 (const-friendly)
Sourcepub const fn as_uint_ref(&self) -> &NonZeroUintRef
pub const fn as_uint_ref(&self) -> &NonZeroUintRef
Borrow this NonZero<Uint> as a &NonZeroUintRef.
Sourceimpl<const LIMBS: usize> NonZero<Int<LIMBS>>
impl<const LIMBS: usize> NonZero<Int<LIMBS>>
Sourcepub const fn new_unwrap(n: Int<LIMBS>) -> Self
pub const fn new_unwrap(n: Int<LIMBS>) -> Self
Sourcepub const fn abs_sign(&self) -> (NonZero<Uint<LIMBS>>, Choice)
pub const fn abs_sign(&self) -> (NonZero<Uint<LIMBS>>, Choice)
The sign and magnitude of this NonZeroInt.
Sourcepub const fn abs(&self) -> NonZero<Uint<LIMBS>>
pub const fn abs(&self) -> NonZero<Uint<LIMBS>>
The magnitude of this NonZeroInt.
Sourceimpl NonZero<BoxedUint>
impl NonZero<BoxedUint>
Sourcepub fn as_uint_ref(&self) -> &NonZeroUintRef
Available on crate feature alloc only.
pub fn as_uint_ref(&self) -> &NonZeroUintRef
alloc only.Borrow this NonZeroBoxedUint as a &NonZeroUintRef.
Sourceimpl<T: ToUnsigned + ?Sized> NonZero<T>
impl<T: ToUnsigned + ?Sized> NonZero<T>
Sourcepub fn to_unsigned(&self) -> NonZero<T::Unsigned>
pub fn to_unsigned(&self) -> NonZero<T::Unsigned>
Convert from a reference into an owned NonZero<T: Unsigned>.
Sourceimpl<T> NonZero<T>
impl<T> NonZero<T>
Sourcepub fn from_be_byte_array(bytes: ByteArray<T>) -> CtOption<Self>
Available on crate feature hybrid-array only.
pub fn from_be_byte_array(bytes: ByteArray<T>) -> CtOption<Self>
hybrid-array only.Decode a non-zero integer from big endian bytes.
Sourcepub fn from_le_byte_array(bytes: ByteArray<T>) -> CtOption<Self>
Available on crate feature hybrid-array only.
pub fn from_le_byte_array(bytes: ByteArray<T>) -> CtOption<Self>
hybrid-array only.Decode a non-zero integer from little endian bytes.
Sourceimpl<const LIMBS: usize> NonZero<Uint<LIMBS>>
impl<const LIMBS: usize> NonZero<Uint<LIMBS>>
Sourcepub const fn gcd_unsigned(&self, rhs: &Uint<LIMBS>) -> Self
pub const fn gcd_unsigned(&self, rhs: &Uint<LIMBS>) -> Self
Compute the greatest common divisor of self and rhs.
Sourcepub const fn gcd_unsigned_vartime(&self, rhs: &Uint<LIMBS>) -> Self
pub const fn gcd_unsigned_vartime(&self, rhs: &Uint<LIMBS>) -> Self
Compute the greatest common divisor of self and rhs.
Executes in variable time w.r.t. all input parameters.
Sourcepub const fn xgcd(&self, rhs: &Self) -> XgcdOutput<LIMBS, NonZeroUint<LIMBS>>
pub const fn xgcd(&self, rhs: &Self) -> XgcdOutput<LIMBS, NonZeroUint<LIMBS>>
Execute the Extended GCD algorithm.
Given (self, rhs), computes (g, x, y) s.t. self * x + rhs * y = g = gcd(self, rhs).
Sourceimpl<const LIMBS: usize> NonZero<Uint<LIMBS>>
impl<const LIMBS: usize> NonZero<Uint<LIMBS>>
Sourcepub const fn floor_root_vartime(&self, exp: NonZeroU32) -> Self
pub const fn floor_root_vartime(&self, exp: NonZeroU32) -> Self
Computes floor(self^(1/exp)).
Callers can check if self is an exact power of exp by exponentiating the result.
This method is variable time in self and in the exponent.
Sourcepub fn checked_root_vartime(&self, exp: NonZeroU32) -> Option<Self>
pub fn checked_root_vartime(&self, exp: NonZeroU32) -> Option<Self>
Compute the root self^(1/exp) returning an Option which is_some
only if the root is exact.
This method is variable time in self and in the exponent.
Sourceimpl<const LIMBS: usize> NonZero<Uint<LIMBS>>
impl<const LIMBS: usize> NonZero<Uint<LIMBS>>
Sourcepub const fn floor_sqrt(&self) -> Self
pub const fn floor_sqrt(&self) -> Self
Computes floor(√(self)) in constant time.
Callers can check if self is a square by squaring the result.
Sourcepub const fn floor_sqrt_vartime(&self) -> Self
pub const fn floor_sqrt_vartime(&self) -> Self
Computes floor(√(self)).
Callers can check if self is a square by squaring the result.
Variable time with respect to self.
Sourcepub fn checked_sqrt(&self) -> CtOption<Self>
pub fn checked_sqrt(&self) -> CtOption<Self>
Perform checked sqrt, returning a CtOption which is_some
only if the square root is exact.
Sourcepub fn checked_sqrt_vartime(&self) -> Option<Self>
pub fn checked_sqrt_vartime(&self) -> Option<Self>
Perform checked sqrt, returning an Option which is_some
only if the square root is exact.