pub struct Odd<T: ?Sized>(/* private fields */);Expand description
Wrapper type for odd integers.
These are frequently used in cryptography, e.g. as a modulus.
Implementations
Sourceimpl<const LIMBS: usize> Odd<Int<LIMBS>>
impl<const LIMBS: usize> Odd<Int<LIMBS>>
Sourcepub const fn gcd_unsigned(&self, rhs: &Uint<LIMBS>) -> Odd<Uint<LIMBS>>
pub const fn gcd_unsigned(&self, rhs: &Uint<LIMBS>) -> Odd<Uint<LIMBS>>
Compute the greatest common divisor of self and rhs.
Sourcepub const fn gcd_unsigned_vartime(&self, rhs: &Uint<LIMBS>) -> OddUint<LIMBS>
pub const fn gcd_unsigned_vartime(&self, rhs: &Uint<LIMBS>) -> OddUint<LIMBS>
Compute the greatest common divisor of self and rhs.
Executes in variable time w.r.t. all input parameters.
Sourceimpl<const LIMBS: usize> Odd<Uint<LIMBS>>
impl<const LIMBS: usize> Odd<Uint<LIMBS>>
Sourcepub const fn from_be_hex(hex: &str) -> Self
pub const fn from_be_hex(hex: &str) -> Self
Sourcepub const fn from_le_hex(hex: &str) -> Self
pub const fn from_le_hex(hex: &str) -> Self
Create a new Odd<Uint<LIMBS>> from the provided little endian hex string.
Panics
- if the hex is malformed or not zero-padded accordingly for the size.
- if the value is even.
Sourcepub const fn as_uint_ref(&self) -> &OddUintRef
pub const fn as_uint_ref(&self) -> &OddUintRef
Borrow this OddUint as a &OddUintRef.
Sourceimpl Odd<UintRef>
impl Odd<UintRef>
Sourcepub const fn to_uint_resize<const T: usize>(&self) -> Odd<Uint<T>>
pub const fn to_uint_resize<const T: usize>(&self) -> Odd<Uint<T>>
Construct an Odd<Uint<T>> from the unsigned integer value,
truncating the upper bits if the value is too large to be
represented.
Sourceimpl Odd<BoxedUint>
impl Odd<BoxedUint>
Sourcepub const fn as_uint_ref(&self) -> &OddUintRef
Available on crate feature alloc only.
pub const fn as_uint_ref(&self) -> &OddUintRef
alloc only.Borrow this OddBoxedUint as a &OddUintRef.
Sourceimpl<const LIMBS: usize> Odd<Uint<LIMBS>>
impl<const LIMBS: usize> Odd<Uint<LIMBS>>
Sourcepub const fn gcd_unsigned(&self, rhs: &Uint<LIMBS>) -> Self
pub const fn gcd_unsigned(&self, rhs: &Uint<LIMBS>) -> Self
Compute the greatest common divisor of self and rhs.
Sourcepub const fn gcd_unsigned_vartime(&self, rhs: &Uint<LIMBS>) -> Self
pub const fn gcd_unsigned_vartime(&self, rhs: &Uint<LIMBS>) -> Self
Compute the greatest common divisor of self and rhs.
Executes in variable time w.r.t. all input parameters.
Sourceimpl Odd<UintRef>
impl Odd<UintRef>
Sourcepub const fn invert_mod_u64(&self) -> u64
pub const fn invert_mod_u64(&self) -> u64
Returns the multiplicative inverse of the argument modulo 2^64. The implementation is based on the Hurchalla’s method for computing the multiplicative inverse modulo a power of two.
For better understanding the implementation, the following paper is recommended: J. Hurchalla, “An Improved Integer Multiplicative Inverse (modulo 2^w)”, https://arxiv.org/pdf/2204.limbs4342.pdf
Variable time with respect to the number of words in value, however that number will be
fixed for a given integer size.