Split

Trait Split 

pub trait Split<const LO: usize> {
    type Output: Integer;
}
Define the result of splitting a number into two parts, with the first part having the width LO.

Required Associated Types

type Output: Integer

High limbs of output: having the width of Self minus LO.

Implementors

Source

impl Split<{ <$name>::LIMBS / 2 }> for U128

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U256

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U384

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U512

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U640

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U768

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U896

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U1152

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U1280

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U1408

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U1536

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U1664

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U1792

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U1920

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U2048

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U2176

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U2304

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U2432

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U2560

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U2688

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U2816

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U2944

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U3072

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U3200

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U3328

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U3456

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U3584

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U3712

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U3840

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U3968

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U4096

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U4224

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U4352

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U4480

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U4608

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U4736

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U4864

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U4992

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U5120

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U5248

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U5376

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U5504

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U5632

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U5760

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U5888

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U6016

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U6144

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U6272

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U6400

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U6528

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U6656

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U6784

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U6912

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U7040

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U7168

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U7296

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U7424

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U7552

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U7680

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U7808

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U7936

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U8064

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U8192

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U8320

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U12288

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ <$name>::LIMBS / 2 }> for U16384

Source

type Output = Uint<{ <$name>::LIMBS / 2 }>

Source

impl Split<{ U64::LIMBS * $size }> for U192

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U192

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U256

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U256

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U320

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U320

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U320

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U320

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U384

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U384

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U384

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U384

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U448

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U448

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U448

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U448

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U448

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U448

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U512

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U512

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U512

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U512

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U512

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U512

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U576

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U576

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U576

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U576

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U576

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U576

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U576

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U576

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U640

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U640

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U640

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U640

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U640

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U640

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U640

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U640

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U704

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U768

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U832

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U896

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U960

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1024

Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1152

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U1152

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U2176

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U2176

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U3200

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U3200

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U4224

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U4224

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U6272

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U6272

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U8320

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>

Source

impl Split<{ U64::LIMBS * $size }> for U8320

Available on crate feature extra-sizes only.
Source

type Output = Uint<{ <$name>::LIMBS - U64::LIMBS * $size }>