pub trait SplitEven {
type Output: Integer;
}Expand description
Define the result of splitting a number into two parts, with each part having an equal width.
Required Associated Types
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".